@@ -21,8 +21,9 @@ bootdata.trans.msm <- function(x) {
     for (j in dat$covlabels.orig) {
         frominds <- seq(1, 2*length(inds)-1, 2)
         data.boot[frominds, j] <- data.boot[frominds+1,j] <- dat$cov.orig[inds, j]
-        if (is.factor(dat$cov.orig[, j]))
-            data.boot[, j] <- factor(data.boot[,j], labels=levels(dat$cov.orig[, j]))
+        if (is.factor(dat$cov.orig[, j])) { 
+            data.boot[, j] <- factor(data.boot[,j], labels=sort(unique(dat$cov.orig[inds, j])))
+        }
     }
     colnames(data.boot) <- gsub("factor\\((.+)\\)", "\\1", colnames(data.boot))
     data.boot
@@ -55,7 +56,7 @@ bootdata.subject.msm <- function(x) {
     for (j in dat$covlabels.orig) { 
         data.boot[, j] <- dat$cov.orig[inds, j]
         if (is.factor(dat$cov.orig[, j]))
-            data.boot[, j] <- factor(data.boot[,j], labels=levels(dat$cov.orig[, j]))
+            data.boot[, j] <- factor(data.boot[,j], labels=sort(unique(dat$cov.orig[, j])))
     }
     colnames(data.boot) <- gsub("factor\\((.+)\\)", "\\1", colnames(data.boot))
     data.boot
@@ -74,17 +75,22 @@ bootdata.subject.msm <- function(x) {
 
 ### a) if call supplied as factor(), strip factor() from name. 
 
+
 boot.msm <- function(x, stat=pmatrix.msm, B=1000, file=NULL){
-    boot.list <- vector(B, mode="list")
     if (!is.null(x$call$subject)) x$call$subject <- substitute(subject.name)
     if (!is.null(x$call$obstype)) x$call$obstype <- substitute(obstype.name)
     if (!is.null(x$call$obstrue)) x$call$obstrue <- substitute(obstrue.name)
-    for (i in 1:B) {
+    boot.fn <- function(dummy){ 
         boot.data <- if (x$hmodel$hidden || x$cmodel$ncens) bootdata.subject.msm(x) else bootdata.trans.msm(x)
         x$call$data <- substitute(boot.data)
-        boot.list[[i]] <- try(eval(x$call))
-        if (!is.null(stat))
-            boot.list[[i]] <- stat(boot.list[[i]])
+        res <- try(eval(x$call))
+        if (!inherits(res, "try-error") && !is.null(stat))
+            res <- try(stat(res))
+        res
+    }
+    boot.list <- vector(B, mode="list")
+    for (i in 1:B) {
+        boot.list[[i]] <- boot.fn()
         if (!is.null(file)) save(boot.list, file=file)
     }
     boot.list
@@ -111,6 +117,13 @@ qratio.ci.msm <- function(x, ind1, ind2, covariates="mean", cl=0.95, B=1000) {
     c(quantile(q.vec, c(0.5 - cl/2, 0.5 + cl/2)), sd(q.vec))
 }
 
+pnext.ci.msm <- function(x, covariates="mean", cl=0.95, B=1000) {
+    p.list <- boot.msm(x, function(x)pnext.msm(x=x, covariates=covariates, ci="none")$estimates, B)
+    p.array <- array(unlist(p.list), dim=c(dim(p.list[[1]]), length(p.list)))
+    p.ci <- apply(p.array, c(1,2), function(x)(quantile(x, c(0.5 - cl/2, 0.5 + cl/2))))
+    aperm(p.ci, c(2,3,1))
+}
+
 pmatrix.ci.msm <- function(x, t, t1, covariates="mean", cl=0.95, B=1000) {
     p.list <- boot.msm(x, function(x)pmatrix.msm(x=x, t=t, t1=t1, covariates=covariates,ci="none"), B)
     p.array <- array(unlist(p.list), dim=c(dim(p.list[[1]]), length(p.list)))
@@ -125,8 +138,14 @@ pmatrix.piecewise.ci.msm <- function(x, t1, t2, times, covariates="mean", cl=0.9
     aperm(p.ci, c(2,3,1))
 }
 
-totlos.ci.msm <- function(x, start=1, end=NULL, fromt=0, tot=Inf, covariates="mean", cl=0.95, B=1000, ...) {
-    t.list <- boot.msm(x, function(x)totlos.msm(x, start, end, fromt, tot, covariates), B)
+totlos.ci.msm <- function(x, start=1, end=NULL, fromt=0, tot=Inf, covariates="mean", cl=0.95, B=1000,...) {
+    t.list <- boot.msm(x, function(x)totlos.msm(x, start, end, fromt, tot, covariates, ci="none",...), B)
+    t.array <- do.call("rbind", t.list)
+    apply(t.array, 2, function(x)(quantile(x, c(0.5 - cl/2, 0.5 + cl/2))))
+}
+
+efpt.ci.msm <- function(x, qmatrix=NULL, tostate, covariates="mean", cl=0.95, B=1000) {
+    t.list <- boot.msm(x, function(x)efpt.msm(x, qmatrix, tostate, covariates, ci="none"), B)
     t.array <- do.call("rbind", t.list)
     apply(t.array, 2, function(x)(quantile(x, c(0.5 - cl/2, 0.5 + cl/2))))
 }
@@ -156,17 +175,19 @@ expected.ci.msm <- function(x,
 
 ### Compute a CI for a statistic using a sample from the assumed MVN
 ### distribution of MLEs of log Q, logit E and covariate effects on these
-### Not user visible: only support a limited set of statistics based on Q matrix and E matrix.
+### Not user visible: only support statistics based on Q matrix and E matrix
+### i.e. statistics computed as functions of x$Qmatrices, x$Ematrices and x$paramdata$params
 
-normboot.msm <- function(x, stat, B=100) {
+normboot.msm <- function(x, stat, B=1000) {
     ## simulate from vector of unreplicated parameters, to avoid numerical problems with rmvnorm when lots of correlations are 1 
     if (!x$foundse) stop("Asymptotic standard errors not available in fitted model")
     sim <- rmvnorm(B, x$opt$par, solve(0.5 * x$opt$hessian))
     params <- matrix(nrow=B, ncol=x$paramdata$npars)  # replicate constrained parameters.
     params[,x$paramdata$optpars] <- sim 
-    params[,x$paramdata$fixedpars] <- x$paramdata$params[x$paramdata$fixedpars]
-    params[,x$paramdata$hmmpars] <- msm.mninvlogit.transform(x$paramdata$params[x$paramdata$hmmpars], x$hmodel$plabs, x$hmodel$parstate)
-    params <- params[, !duplicated(abs(x$paramdata$constr))][, abs(x$paramdata$constr)]*rep(sign(x$paramdata$constr), each=B)
+    params[,x$paramdata$fixedpars] <- rep(x$paramdata$params[x$paramdata$fixedpars], each=B)
+    params[,x$paramdata$hmmpars] <- rep(msm.mninvlogit.transform(x$paramdata$params[x$paramdata$hmmpars], x$hmodel$plabs, x$hmodel$parstate), each=B)
+    params <- params[, !duplicated(abs(x$paramdata$constr)), drop=FALSE][, abs(x$paramdata$constr), drop=FALSE] *
+        rep(sign(x$paramdata$constr), each=B)
 
     sim.stat <- vector(B, mode="list")
     for (i in 1:B) {
@@ -198,11 +219,18 @@ qematrix.normci.msm <- function(x, covariates="mean", intmisc="intens", sojourn=
 }
 
 qratio.normci.msm <- function(x, ind1, ind2, covariates="mean", cl=0.95, B=1000) {
-    q.list <- normboot.msm(x, function(x)qratio.msm(x=x, ind1=ind1, ind2=ind2, covariates=covariates)["estimate"], B)
+    q.list <- normboot.msm(x, function(x)qratio.msm(x=x, ind1=ind1, ind2=ind2, covariates=covariates, ci="none")["estimate"], B)
     q.vec <- unlist(q.list)
     c(quantile(q.vec, c(0.5 - cl/2, 0.5 + cl/2)), sd(q.vec))
 }
 
+pnext.normci.msm <- function(x, covariates="mean", cl=0.95, B=1000) {
+    p.list <- normboot.msm(x, function(x)pnext.msm(x=x, covariates=covariates, ci="none")$estimates, B)
+    p.array <- array(unlist(p.list), dim=c(dim(p.list[[1]]), length(p.list)))
+    p.ci <- apply(p.array, c(1,2), function(x)(quantile(x, c(0.5 - cl/2, 0.5 + cl/2))))
+    aperm(p.ci, c(2,3,1))
+}
+
 pmatrix.normci.msm <- function(x, t, t1, covariates="mean", cl=0.95, B=1000) {
     p.list <- normboot.msm(x, function(x)pmatrix.msm(x=x, t=t, t1=t1, covariates=covariates, ci="none"), B)
     p.array <- array(unlist(p.list), dim=c(dim(p.list[[1]]), length(p.list)))
@@ -218,7 +246,13 @@ pmatrix.piecewise.normci.msm <- function(x, t1, t2, times, covariates="mean", cl
 }
 
 totlos.normci.msm <- function(x, start=1, end=NULL, fromt=0, tot=Inf, covariates="mean", cl=0.95, B=1000, ...) {
-    t.list <- normboot.msm(x, function(x)totlos.msm(x, start, end, fromt, tot, covariates, ci="none"), B)
+    t.list <- normboot.msm(x, function(x)totlos.msm(x, start, end, fromt, tot, covariates, ci="none", ...), B)
+    t.array <- do.call("rbind", t.list)
+    apply(t.array, 2, function(x)(quantile(x, c(0.5 - cl/2, 0.5 + cl/2))))
+}
+
+efpt.normci.msm <- function(x, qmatrix=NULL, tostate, covariates="mean", cl=0.95, B=1000, ...) {
+    t.list <- normboot.msm(x, function(x)efpt.msm(x, qmatrix, tostate, covariates, ci="none", ...), B)
     t.array <- do.call("rbind", t.list)
     apply(t.array, 2, function(x)(quantile(x, c(0.5 - cl/2, 0.5 + cl/2))))
 }
