@@ -14,6 +14,7 @@
 #define NODEBUG
 #define NODEBUG2
 #define NOVITDEBUG
+#define NOFISHDEBUG
 
 linkfn LINKFNS[3][2] = {
     {identity, identity},
@@ -37,7 +38,8 @@ hmmfn HMODELS[] = {
     hmmMETNorm,
     hmmMEUnif,
     hmmNBinom,
-    hmmBeta
+    hmmBeta,
+    hmmT
 };
 
 #define OBS_SNAPSHOT 1
@@ -216,6 +218,9 @@ void update_likhidden(double *curr, int nc, int obsno, msmdata *d, qmodel *qm, q
 	 (d->obstype[obsno] == OBS_EXACT), qm->analyticp, qm->iso, qm->perm, qm->qperm, 0);
     for(j = 0; j < qm->nst; ++j)
 	{
+#ifdef DEBUG2
+	    printf("pmat: ");
+#endif
 	    newp[j] = 0.0;
 	    for(i = 0; i < qm->nst; ++i)
 		{
@@ -283,11 +288,24 @@ double likhidden(int pt, /* ordinal subject ID */
     /* Transform initp from probs relative to state 1 prob back to absolute probs */
     relative2absolutep(newinitp, newinitp, qm->nst, 0);
     /* Likelihood contribution for initial observation */
+#ifdef DEBUG2
+    printf("pout: ");
+    for (i = 0; i < qm->nst; ++i) {
+	printf("%lf, ", pout[i]);
+    }
+      printf("\ncump: ");
+#endif
     for (i = 0; i < qm->nst; ++i) {
       /* Ignore initprobs if observation is known to be the true state  */
       if (d->obstrue[d->firstobs[pt]]) newinitp[i] = 1;
       cump[i] = pout[i] * newinitp[i];
+#ifdef DEBUG2
+      printf("%lf, ", cump[i]);
+#endif
     }
+#ifdef DEBUG2
+      printf("\n");
+#endif
     lweight=0;
     /* Matrix product loop to accumulate the likelihood for subsequent observations */
     for (obsno = d->firstobs[pt]+1; obsno <= d->firstobs[pt+1] - 1; ++obsno)
@@ -456,7 +474,7 @@ void Viterbi(msmdata *d, qmodel *qm, qcmodel *qcm,
     i = 0;
     if (d->obstrue[i]) {
       for (k = 0; k < qm->nst; ++k)
-	lvold[k] = (k+1 == d->obs[i] ? 1 : R_NegInf);
+	lvold[k] = (k+1 == d->obs[i] ? 0 : R_NegInf);
     }
     else {
       GetCensored(d->obs[i], cm, &nc, &curr);
@@ -464,7 +482,7 @@ void Viterbi(msmdata *d, qmodel *qm, qcmodel *qcm,
       if (nc > 1) {
 	for (k = 0, j = 0; k < qm->nst; ++k) {
 	  if (k+1 == curr[j]) {
-	    lvold[k] = 1;
+	    lvold[k] = 0;
 	    ++j;
 	  }
 	  else lvold[k] = R_NegInf;
@@ -489,7 +507,7 @@ void Viterbi(msmdata *d, qmodel *qm, qcmodel *qcm,
 	    if ((i < d->nobs) && (d->subject[i] == d->subject[i-1]))
 		{
 #ifdef VITDEBUG
-		    printf("subject %d\n ", d->subject[i]);
+		    printf("subject %d\n", d->subject[i]);
 #endif
 		    dt = d->time[i] - d->time[i-1];
 		    AddCovs(i - 1, d->nobs, qm->npars, qcm->ncovs, qm->intens, newintens,
@@ -506,33 +524,26 @@ void Viterbi(msmdata *d, qmodel *qm, qcmodel *qcm,
 		    GetOutcomeProb(pout, curr, nc, newpars, hm, qm, d->obstrue[i]);
 #ifdef VITDEBUG
 		    for (tru=0;tru<nc;++tru) printf("curr[%d] = %1.0lf, ",tru, curr[tru]); printf("\n");
-		    for (k=0; k<qm->npars; ++k) printf("intens[%d] = %f", k, qm->intens[k]); printf("\n");
-		    for (k=0; k<qcm->ncovs[0]; ++k) printf("cov[%d] = %f", k, d->cov[MI(i-1,k,d->nobs)]); printf("\n");
-		    for (k=0; k<qm->npars; ++k) printf("newintens[%d] = %f", k, newintens[k]); printf("\n");
-		    for (j = 0; j < qm->nst; ++j) {
-		      for (k=0; k<hm->npars[j]; ++k) printf("hpar[%d] = %f", k, hm->pars[k]);
-		      printf("\n");
-		    }
 #endif
 		    Pmat(pmat, dt, newintens, qm->npars, qm->ivector, qm->nst,
 			 (d->obstype[i] == OBS_EXACT), qm->analyticp, qm->iso, qm->perm,  qm->qperm, 0);
 
 		    for (tru = 0; tru < qm->nst; ++tru)
 			{
-			  if (d->obstrue[i-1]) {
-			    lvnew[tru] = (tru == d->obs[i-1] - 1 ? 1 : R_NegInf);
-			    ptr[MI(i, tru, d->nobs)] = kmax = d->obs[i-1] - 1;
-			  }
-			  else {
-			    for (k = 0; k < qm->nst; ++k)
+/* lvnew =  log prob of most likely path ending in tru at current obs.
+   kmax  = most likely state at the previous obs
+*/
+			    for (k = 0; k < qm->nst; ++k) {
 				lvp[k] = lvold[k] + log(pmat[MI(k, tru, qm->nst)]);
-			    pmax(lvp, qm->nst, &kmax);
+			    }
+			    if (d->obstrue[i-1])
+				kmax = d->obs[i-1] - 1;
+			    else pmax(lvp, qm->nst, &kmax);
 			    lvnew[tru] = log ( pout[tru] )  +  lvp[kmax];
 			    ptr[MI(i, tru, d->nobs)] = kmax;
-			  }
 #ifdef VITDEBUG
-			    printf("true %d, pout[%d] = %lf, lvold = %lf, pmat = %lf, lvnew = %lf, mi = %d, ptr=%d\n",
-				   tru, tru, pout[tru], lvold[tru], pmat[MI(kmax, tru, qm->nst)], lvnew[tru], MI(i, tru, d->nobs), ptr[MI(i, tru, d->nobs)]);
+			    printf("true %d, pout[%d] = %lf, lvold = %lf, pmat = %lf, lvnew = %lf, ptr[%d,%d]=%d\n",
+				   tru, tru, pout[tru], lvold[tru], pmat[MI(kmax, tru, qm->nst)], lvnew[tru], i, tru, ptr[MI(i, tru, d->nobs)]);
 #endif
 			}
 		    for (k = 0; k < qm->nst; ++k)
@@ -540,29 +551,30 @@ void Viterbi(msmdata *d, qmodel *qm, qcmodel *qcm,
 		}
 	    else
 		{
-#ifdef VITDEBUG
-		    printf("traceback for subject %d\n ", d->subject[i-1]);
-#endif
 		    /* Traceback for current individual */
 		    pmax(lvold, qm->nst, &kmax);
-#ifdef VITDEBUG
-		    printf("kmax = %d\n", kmax);
-#endif
 		    obs = i-1;
-		    fitted[obs] = (d->obstrue[obs] ? d->obs[obs]-1 : kmax);  /* if last obs censoring, then kmax=0 since pout all 0 for prev obs, wrong, should be 0,1,1,0. */
+		    fitted[obs] = (d->obstrue[obs] ? d->obs[obs]-1 : kmax);
+#ifdef VITDEBUG
+		    printf("traceback for subject %d\n", d->subject[i-1]);
+		    printf("obs=%d,fitted[%d]=%1.0lf\n",obs,obs,fitted[obs]);
+#endif
 		    while   ( (obs > 0) && (d->subject[obs] == d->subject[obs-1]) )
 			{
 			    fitted[obs-1] = ptr[MI(obs, fitted[obs], d->nobs)];
 #ifdef VITDEBUG
-			    printf("mi = %d, ptr %d = %1.0lf, ", MI(obs, (int) fitted[obs], d->nobs), obs-1, fitted[obs-1]);
+			    printf("fitted[%d] = ptr[%d,%1.0lf] = %1.0lf\n", obs-1, obs, fitted[obs], fitted[obs-1]);
 #endif
 			    --obs;
 			}
+#ifdef VITDEBUG
+		    printf("\n");
+#endif
 
 		    /* Add covariate effects on initp (expressed as p(cat) / p(baseline cat)) */
 		    if (d->obstrue[i]) {
 		      for (k = 0; k < qm->nst; ++k)
-			lvold[k] = (k+1 == d->obs[i] ? 1 : R_NegInf);
+			lvold[k] = (k+1 == d->obs[i] ? 0 : R_NegInf);
 		    }
 		    else {
 		      GetCensored(d->obs[i], cm, &nc, &curr);
@@ -570,7 +582,7 @@ void Viterbi(msmdata *d, qmodel *qm, qcmodel *qcm,
 		      if (nc > 1) {
 			for (k = 0, j = 0; k < qm->nst; ++k) {
 			  if (k+1 == curr[j]) {
-			    lvold[k] = 1;
+			    lvold[k] = 0;
 			    ++j;
 			  }
 			  else lvold[k] = R_NegInf;
@@ -630,17 +642,18 @@ void msmLikelihood (msmdata *d, qmodel *qm, qcmodel *qcm,
    multi-state Markov model. */
 
 void derivsimple(msmdata *d, qmodel *qm, qcmodel *qcm,
-		 cmodel *cm, hmodel *hm, double *deriv)
+		 cmodel *cm, hmodel *hm, double *deriv, int do_info, double *info)
 {
-    int i, p, np=qm->npars, ndp=qm->ndpars, ndc=qcm->ndpars, totcovs=0;
-    double contrib=0;
-
-    double *dcontrib = Calloc(ndp+ndc, double);
+    int i, j, p, q, np=qm->npars, ndp=qm->ndpars, ndc=qcm->ndpars, totcovs=0;
+    double *pm = Calloc(qm->nst, double);
+    double *dpm = Calloc(qm->nst * (ndp+ndc), double);
     double *dpmat = Calloc(qm->nst * qm->nst * (ndp+ndc), double);
     double *pmat = Calloc(qm->nst * qm->nst, double);
     double *newintens = Calloc(np, double);
     double *x = Calloc(qcm->ncovs[0], double);
-
+#ifdef FISHDEBUG
+    double dptest;
+#endif
     for (i=0; i < d->nobs; ++i)
 	{
 	    R_CheckUserInterrupt();
@@ -655,27 +668,116 @@ void derivsimple(msmdata *d, qmodel *qm, qcmodel *qcm,
 		      qm->constr, qcm->constr, qcm->wcov, (d->obstype[i] == OBS_EXACT));
 	    }
 	    if (d->obstype[i] == OBS_DEATH) {
-		contrib = pijdeath(d->fromstate[i], d->tostate[i], pmat, newintens, qm->ivector, qm->nst);
+		for (j=0; j < qm->nst; ++j)
+		    pm[j] = pijdeath(d->fromstate[i], j, pmat, newintens, qm->ivector, qm->nst);
 		dpijdeath(d->fromstate[i], d->tostate[i], x, dpmat, pmat, newintens, qm->intens, qm->ivector,
-			  qm->nst, qm->constr, qcm->constr, ndp, ndc, qcm->ncovs[0], dcontrib);
+			  qm->nst, qm->constr, qcm->constr, ndp, ndc, qcm->ncovs[0], dpm);
 	    }
 	    else {
-		contrib = pmat[MI(d->fromstate[i], d->tostate[i], qm->nst)];
-		for (p = 0; p < ndp+ndc; ++p)
-		    dcontrib[p] = dpmat[MI3(d->fromstate[i], d->tostate[i], p, qm->nst, qm->nst)];
+		for (j=0; j < qm->nst; ++j)  {
+		    pm[j] = pmat[MI(d->fromstate[i], j, qm->nst)];
+		    for (p = 0; p < ndp+ndc; ++p)
+			dpm[MI(j,p,qm->nst)] = dpmat[MI3(d->fromstate[i], j, p, qm->nst, qm->nst)];
+		}
 	    }
+
 	    for (p = 0; p < ndp+ndc; ++p) {
-		deriv[p] += d->nocc[i] * dcontrib[p] / contrib;
+		deriv[p] += d->nocc[i] * dpm[MI(d->tostate[i],p,qm->nst)] / pm[d->tostate[i]];
 	    }
 #ifdef DEBUG
 	    for (p = 0; p < ndp+ndc; ++p) {
-	      printf("%d, %d, %d, %d, %6.4f, %d, %d, %lf\n", i, p, d->fromstate[i], d->tostate[i], d->timelag[i], d->obstype[i], d->nocc[i], -2 * d->nocc[i] * dcontrib[p] / contrib);
+		printf("%d, %d, %d, %d, %6.4f, %d, %d, %lf\n", i, p, d->fromstate[i], d->tostate[i], d->timelag[i], d->obstype[i], d->nocc[i], -2 * d->nocc[i] * dpm[MI(d->tostate[i],p,qm->nst)] / pm[d->tostate[i]]);
 	    }
 #endif
+	    if (do_info)  {  /* Fisher information */
+		if ((i==0) || (d->whicha[i] != d->whicha[i-1]) || (d->obstype[i] != d->obstype[i-1]) ||
+		    (d->fromstate[i] != d->fromstate[i-1])) {
+#ifdef FISHDEBUG
+		    for(j = 0; j<qm->nst; ++j)  {
+			printf("dpm[%d,%d,%d]= ",d->fromstate[i],j,i);
+			for (p = 0; p < ndp+ndc; ++p) {
+			    dptest = dpm[MI(j,p,qm->nst)];
+			    if (p < 2) dptest = dptest * newintens[p];
+			    printf("%lf ", dptest);
+			}
+			printf("\n");
+		    }
+		    printf("pmat[%d,%d]= ",d->fromstate[i],i);
+		    for(j = 0; j<qm->nst; ++j)  {
+			printf("%lf ",pm[j]);
+		    }
+		    printf("\n");
+#endif
+
+		    for (p = 0; p < ndp+ndc; ++p) {
+			for (q = 0; q < ndp+ndc; ++q) {
+			    /* for expected information, sum over all possible destination states for this fromstate/time/cov combination */
+			    for(j = 0; j<qm->nst; ++j)  {
+				if (pm[j] > 0)
+				    info[MI(p,q,ndp+ndc)] +=  d->noccsum[i] * dpm[MI(j,p,qm->nst)] * dpm[MI(j,q,qm->nst)] /  pm[j];
+#ifdef FISHDEBUG
+				if (p==0 && q==0)
+				    printf("%lf\n",d->noccsum[i] * dpm[MI(j,p,qm->nst)] * dpm[MI(j,q,qm->nst)] /  pm[j] * newintens[p]* newintens[p]);
+#endif
+			    }
+			}
+		    }
+		}
+	    }
 	}
-    for (p = 0; p < ndp+ndc; ++p)
-	deriv[p] *= -2;
-    Free(dcontrib); Free(dpmat); Free(pmat); Free(newintens); Free(x);
+    for (p = 0; p < ndp+ndc; ++p) {
+	deriv[p] *= -2;   /* above is dlogL/dtu as in Kalb+Law, we want
+			     deriv of -2 loglik  */
+	if (do_info) {
+	    for (q = 0; q < ndp+ndc; ++q) {
+		info[MI(p,q,ndp+ndc)] *= 2; /* above is E(-d2logL/dtudtv) as in Kalb+Law.
+					       we want second deriv of of -2 loglik */
+	    }
+	}
+    }
+    Free(pm); Free(dpm); Free(dpmat); Free(pmat); Free(newintens); Free(x);
+}
+
+/* Derivatives of the P matrix, used for the Pearson test p-value */
+/* Returns a ntrans * ntostates * npars matrix */
+/* Panel data only (obstype 1) */
+
+void dpmat_obs(msmdata *d, qmodel *qm, qcmodel *qcm,
+		 cmodel *cm, hmodel *hm, double *deriv)
+{
+    int pt, i, j, k, p, from, to, np=qm->npars, ndp=qm->ndpars, ndc=qcm->ndpars, totcovs=0;
+    double *dpmat = Calloc(qm->nst * qm->nst * (ndp+ndc), double);
+    double *newintens = Calloc(np, double);
+    double *x = Calloc(qcm->ncovs[0], double);
+    double dt;
+
+    j=0;
+    for (pt = 0;  pt < d->npts; ++pt)
+	{
+	    R_CheckUserInterrupt();
+	    if (d->firstobs[pt+1] > d->firstobs[pt] + 1) { /* individual has more than one observation? */
+		for (i = d->firstobs[pt]+1; i < d->firstobs[pt+1]; ++i) {
+		    ++j;
+		    GetCovData(i, d->covobs, d->whichcov, x, qcm->ncovs[0], d->n);
+		    AddCovs(i, d->n, np, qcm->ncovs, qm->intens, newintens,
+			    qcm->coveffect, d->covobs, d->whichcov, &totcovs, log, exp);
+		    dt = d->time[i] - d->time[i-1];
+		    from = fprec(d->obs[i-1] - 1, 0); /* convert state outcome to integer */
+		    to = fprec(d->obs[i] - 1, 0);
+		    DPmat(dpmat, dt, x, newintens, qm->intens, qm->ivector, qm->nst, np, ndp, ndc,
+			  qm->constr, qcm->constr, qcm->wcov, (d->obstype[i] == OBS_EXACT));
+//		    printf("pt %d, obs %d, trans %d, from %d, to %d, ", pt, i, j, from, to);
+		    for (p = 0; p < ndp+ndc; ++p) {
+			for (k=0; k < qm->nst; ++k) {
+			    deriv[MI3(j-1,k,p,d->ntrans,qm->nst)] = dpmat[MI3(from, k, p, qm->nst, qm->nst)];
+//			    printf("%d %lf, ",MI3(j-1,k,p,d->ntrans,qm->nst),dpmat[MI3(from, k, p, qm->nst, qm->nst)]);
+			}
+		    }
+//		    printf("\n");
+		}
+	    }
+	}
+    Free(dpmat); Free(newintens); Free(x);
 }
 
 /* First derivatives of the likelihood for the non-hidden multi-state
@@ -688,12 +790,12 @@ void derivsimple_subj(msmdata *d, qmodel *qm, qcmodel *qcm,
 			cmodel *cm, hmodel *hm, double *deriv)
 {
     int pt, i, p, np=qm->npars, ndp=qm->ndpars, ndc=qcm->ndpars, totcovs=0;
-    double *dcontrib = Calloc(ndp+ndc, double);
+    double *dpm = Calloc(ndp+ndc, double);
     double *dpmat = Calloc(qm->nst * qm->nst * (ndp+ndc), double);
     double *pmat = Calloc(qm->nst * qm->nst, double);
     double *newintens = Calloc(np, double);
     double *x = Calloc(qcm->ncovs[0], double);
-    double contrib=0, dt;
+    double pm=0, dt;
     int from, to;
     for (pt = 0;  pt < d->npts; ++pt){
 	{
@@ -702,31 +804,31 @@ void derivsimple_subj(msmdata *d, qmodel *qm, qcmodel *qcm,
 	      for (p = 0; p < ndp+ndc; ++p) {
 		deriv[MI(pt,p,d->npts)] = 0;
 	      }
-		for (i = d->firstobs[pt]+1; i < d->firstobs[pt+1]; ++i) {
-		    GetCovData(i, d->covobs, d->whichcov, x, qcm->ncovs[0], d->n);
+	      for (i = d->firstobs[pt]+1; i < d->firstobs[pt+1]; ++i) {
+		  GetCovData(i, d->covobs, d->whichcov, x, qcm->ncovs[0], d->n);
 		    AddCovs(i, d->n, np, qcm->ncovs, qm->intens, newintens,
 			    qcm->coveffect, d->covobs, d->whichcov, &totcovs, log, exp);
 
 		    dt = d->time[i] - d->time[i-1];
 		    from = fprec(d->obs[i-1] - 1, 0); /* convert state outcome to integer */
 		    to = fprec(d->obs[i] - 1, 0);
-		    Pmat(pmat, dt, newintens, qm->npars, qm->ivector, qm->nst, (d->obstype[i] == OBS_EXACT), qm->analyticp, qm->iso, qm->perm,  qm->qperm, 0);
+		    Pmat(pmat, dt, newintens, np, qm->ivector, qm->nst, (d->obstype[i] == OBS_EXACT), qm->analyticp, qm->iso, qm->perm,  qm->qperm, 0);
 		    DPmat(dpmat, dt, x, newintens, qm->intens, qm->ivector, qm->nst, np, ndp, ndc,
 			  qm->constr, qcm->constr, qcm->wcov, (d->obstype[i] == OBS_EXACT));
 		    if (d->obstype[i] == OBS_DEATH) {
-			contrib = pijdeath(from, to, pmat, newintens, qm->ivector, qm->nst);
+			pm = pijdeath(from, to, pmat, newintens, qm->ivector, qm->nst);
 			dpijdeath(from, to, x, dpmat, pmat, newintens, qm->intens, qm->ivector,
-				  qm->nst, qm->constr, qcm->constr, ndp, ndc, qcm->ncovs[0], dcontrib);
+				  qm->nst, qm->constr, qcm->constr, ndp, ndc, qcm->ncovs[0], dpm);
 		    }
 		    else {
-			contrib = pmat[MI(from, to, qm->nst)];
+			pm = pmat[MI(from, to, qm->nst)];
 			for (p = 0; p < ndp+ndc; ++p)
-			    dcontrib[p] = dpmat[MI3(from, to, p, qm->nst, qm->nst)];
+			    dpm[p] = dpmat[MI3(from, to, p, qm->nst, qm->nst)];
 		    }
 		    for (p = 0; p < ndp+ndc; ++p) {
-		      deriv[MI(pt,p,d->npts)] += dcontrib[p] / contrib; /* on loglik scale not -2*loglik */
+		      deriv[MI(pt,p,d->npts)] += dpm[p] / pm; /* on loglik scale not -2*loglik */
 #ifdef DEBUG
-		      printf("%d, %d, %d, %d, %d, %6.4f, %d, %lf, %lf\n", i, p, pt, from, to, dt, d->obstype[i], -2 * dcontrib[p] / contrib, -2*deriv[MI(pt,p,d->npts)]);
+		      printf("%d, %d, %d, %d, %d, %6.4f, %d, %lf, %lf\n", i, p, pt, from, to, dt, d->obstype[i], -2 * dpm[p] / pm, -2*deriv[MI(pt,p,d->npts)]);
 #endif
 		    }
 		}
@@ -737,7 +839,7 @@ void derivsimple_subj(msmdata *d, qmodel *qm, qcmodel *qcm,
 		deriv[MI(pt,p,d->npts)] = 0;
 	}
     }
-    Free(dcontrib); Free(dpmat); Free(pmat); Free(newintens); Free(x);
+    Free(dpm); Free(dpmat); Free(pmat); Free(newintens); Free(x);
 }
 
 /* Derivative of log-likelihood. Not available for hidden models or
@@ -747,7 +849,16 @@ void msmDLikelihood (msmdata *d, qmodel *qm, qcmodel *qcm,
 		     cmodel *cm, hmodel *hm,
 		     double *returned)
 {
-    derivsimple (d, qm, qcm, cm, hm, returned);
+    derivsimple (d, qm, qcm, cm, hm, returned, 0, 0);
+}
+
+/* Fisher information matrix */
+
+void msmInfoLikelihood (msmdata *d, qmodel *qm, qcmodel *qcm,
+			cmodel *cm, hmodel *hm,
+			double *returned_deriv, double *returned_info)
+{
+    derivsimple (d, qm, qcm, cm, hm, returned_deriv, 1, returned_info);
 }
 
 void msmDLikelihood_subj (msmdata *d, qmodel *qm, qcmodel *qcm,
@@ -757,6 +868,15 @@ void msmDLikelihood_subj (msmdata *d, qmodel *qm, qcmodel *qcm,
     derivsimple_subj (d, qm, qcm, cm, hm, returned);
 }
 
+/* Derivatives of the P matrix.  Used by Pearson test */
+
+void msmDPmat (msmdata *d, qmodel *qm, qcmodel *qcm,
+		     cmodel *cm, hmodel *hm,
+		     double *returned)
+{
+    dpmat_obs (d, qm, qcm, cm, hm, returned);
+}
+
 /* This function is called from R to provide an entry into C code for
    evaluating likelihoods, derivatives and doing Viterbi state reconstruction  */
 
@@ -777,6 +897,7 @@ void msmCEntry(
 	       double *covobsvec,  /* vectorised matrix of covariate data (by observation) used when calculating derivs by individual */
 	       int *whichcov,    /* which column in the covariate data each cov corresponds to */
 	       int *nocc,       /* Number of occurrences of each distinct combination */
+	       int *noccsum,    /* Number of those summed over tostate and replicated (for Fisher info) */
 	       int *whicha,   /* indicator for the from, to, time lag, covs combination corresponding to the current obs */
 	       int *obstype,   /* observation scheme, 1 snapshot, 2 exact, 3 death */
 
@@ -812,6 +933,7 @@ void msmCEntry(
 	       int *nobs,         /* number of observations in data set (hmm/cens) or number of aggregated transitions (standard) */
 	       int *n,         /* number of observations in data set (used for derivs by individual in standard models. no of rows of covobsvec) */
 	       int *npts,         /* number of individuals in data set */
+	       int *ntrans,         /* number of (disaggregated) transitions, equal to n - npts if at least two obs (one trans) per individual */
 	       int *ncovs,        /* number of covariates on transition rates */
 
 	       int *ncens,     /* number of distinct forms of censoring */
@@ -823,16 +945,18 @@ void msmCEntry(
 	       int *cconstraint, /* constraints for covariates. needed to calculate derivs */
 	       int *whichcovd,  /* which covariate each _distinct_ covariate parameter corresponds to */
 
-	       double *returned   /* returned -2 log likelihood , Viterbi fitted values, or predicted values */
+	       double *returned,   /* returned -2 log likelihood , derivatives or Viterbi fitted values */
+	       double *returned2   /* Fisher information if required */
 	       )
 {
   msmdata d; qmodel qm; qcmodel qcm; cmodel cm; hmodel hm;
 
   d.fromstate = fromstate;     d.tostate = tostate;     d.timelag = timelag;
-  d.cov = covvec;  d.covobs = covobsvec;  d.nocc = nocc;  d.whicha = whicha; d.obstype = obstype; d.obstrue = obstrue;
+  d.cov = covvec;  d.covobs = covobsvec;  d.nocc = nocc;  d.noccsum=noccsum;
+  d.whicha = whicha; d.obstype = obstype; d.obstrue = obstrue;
   d.whichcov = whichcov;  d.whichcovh = whichcovh;  d.whichcovi=whichcovi;
   d.subject = subjvec; d.time = timevec; d.obs = obsvec; d.firstobs = firstobs;
-  d.nobs = *nobs;  d.n = *n; d.npts = *npts;
+  d.nobs = *nobs;  d.n = *n; d.npts = *npts; d.ntrans = *ntrans;
 
   qm.nst = *nst; qm.npars = *nintens; qm.ndpars = *ndintens; qm.ivector = qvector; qm.intens = intens;
   qm.analyticp = *analyticp; qm.iso = *iso; qm.perm = perm; qm.qperm = qperm; qm.constr = qconstraint;
@@ -855,9 +979,18 @@ void msmCEntry(
   }
 
   else if (*do_what == 2) {
-    Viterbi(&d, &qm, &qcm, &cm, &hm, returned);
+      msmInfoLikelihood(&d, &qm, &qcm, &cm, &hm, returned, returned2);
   }
+
   else if (*do_what == 3) {
     msmDLikelihood_subj(&d, &qm, &qcm, &cm, &hm, returned); /* derivative of loglik by subject. used for score residuals */
   }
+
+  else if (*do_what == 4) {
+    Viterbi(&d, &qm, &qcm, &cm, &hm, returned);
+  }
+
+  else if (*do_what == 5) {
+    msmDPmat(&d, &qm, &qcm, &cm, &hm, returned);
+  }
 }
